/*
 * File:   spi.c
 * Author: CYRAX
 *
 * MAX3100 / SPI
 */
#include <htc.h>        /* HiTech General Include File */
#include "main.h"
#include "spi.h"

#define MODEC RIE         /* Receive int enable */
#define MODED BR192        /* Set Baudrate: 19200 bit/s */

#define SPIRATE (0 << SPR1) | (0 << SPR0)            /* SPI rate: fast */
#define SPIMODE (1 << SPE) | (1 << MSTR) | SPIRATE   /* SPI mode */

//   Initialize MAX3100 serial I/O
void MAX3100_init(void)
	{
	initspi();	/* init 16F877 SPI interface */
	uart_write(WRCFG | (MODEC), MODED); /* set UART mode */
	while(MAX3100_rxrdy())    /* flush UART receiver */
		MAX3100_receive();
	}

// Init SPI controller
void initspi(void)
{       
        	 /*View PIC Datasheet Section "SYNC SERIAL PORT CONTROL REGISTER" */
        MAX3100_CS = 1;	/* disable MAX3100 chips */
       
	SSPEN = 0;	/* SSPEN: Synchronous Serial Port Enable bit
                         * 1 = Enables serial port and configures SCK, SDO, SDI, and SS as the source of the serial port pins
                         * 0 = Disables serial port and configures these pins as I/O port pins if it was enabled */

        SSPCON = 0x01;	/* Synchronous Serial Port Mode Select bits
                         * 0x00 -> SPI master, Idle low CKP = 0, Fosc/4
                         * 0x01 -> SPI master, Idle low CKP = 0, Fosc/16
                         * 0x02 -> SPI master, Idle low, Fosc/64 */

        SSPEN = 1;      /* SSPEN: Synchronous Serial Port Enable bit
                         * 1 = Enables serial port and configures SCK, SDO, SDI, and SS as the source of the serial port pins
                         * 0 = Disables serial port and configures these pins as I/O port pins if it was enabled */

        SSPSTAT = 0x00;	/* 0x00 clear status register
                         * 0x40 clear status register, SMP = 0, CKE = 1 */

        SSPIF = 0;	/* Synchronous Serial Port (SSP) Interrupt Flag - clear irq flag
                         * 0 = No SSP interrupt condition has occurred.
                         * 1 = The SSP interrupt condition has occurred, and must be cleared in 
                         * software before returning from the Interrupt Service Routine. The 
                         * conditions that will set this bit are:
                         * ? SPI - A transmission/reception has taken place. */

        /* Synchronous Serial Port Mode
                         * - 0000 = SPI Master mode, clock = FOSC/4
                         * - 0001 = SPI Master mode, clock = FOSC/16
                         * - 0010 = SPI Master mode, clock = FOSC/64
                         * - 0011 = SPI Master mode, clock = TMR2 output/2  */
        /* Slave Mode SS pin Disabled FOSC/16 */
        SSPM0 = 1;     
  	SSPM1 = 0;
  	SSPM2 = 0;
  	SSPM3 = 0;
        
  	SSPEN = 1;   //SPI port pins enabled

        CKP = 0;   /* 0 = Idle state for clock is a low level
                    * 1 = Idle state for clock is a high level */
}

//   Write to MAX3100 UART
static void uart_write(unsigned char cbyte, unsigned char dbyte)
	{
	MAX3100_CS = 0; /* enable MAX3100 chip */
	txrxspi(cbyte);
	txrxspi(dbyte);
	MAX3100_CS = 1; /* disable MAX3100 chip */
	}

//   Read from MAX3100 UART
static void uart_read(unsigned char cbyte, unsigned char *resph, unsigned char *respl)
	{
	MAX3100_CS = 0; /* enable MAX3100 chip */
        *resph = txrxspi(cbyte);
	*respl = txrxspi(0);
	MAX3100_CS = 1; /* disable MAX3100 chip */
	}

// Transmit and Receive SPI
unsigned char txrxspi (unsigned char txc)
{
	SSPBUF = txc;		/* start transmission */
	while (!BF)
	  ;			/* Wait until ready */

	if (WCOL || SSPOV)
		{
		WCOL = 0;
		SSPOV = 0;
		}

return (SSPBUF);
}

//   Check for receiver ready
unsigned char MAX3100_rxrdy(void)
	{
	return (uart_status() & RXRDY) != 0;
	}

//   Check for transmitter ready
unsigned char MAX3100_txrdy(void)
	{
	return (uart_status() & TXRDY) != 0;
	}

//   Get MAX3100 UART status
static unsigned char uart_status(void)
	{
	unsigned char cfgc, cfgd;

	uart_read(RDCFG, &cfgc, &cfgd);
	return cfgc;

}

//   Receive a character
unsigned char MAX3100_receive(void)
	{
	unsigned char sts, ch;

	while (!MAX3100_rxrdy())
		;

	uart_read(RDDAT, &sts, &ch);
	return ch;
	}

void putst(register const char *str)
{
	while((*str)!=0)
	{
		MAX3100_send(*str);
    if (*str==13) MAX3100_send(10);
    if (*str==10) MAX3100_send(13);
		str++;
	}
}

//   Send a character
void MAX3100_send(unsigned char ch)
	{
	while (!MAX3100_txrdy())
		;

	uart_write(WRDAT | RTS, ch);
	}